// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     DateTimeDialogUnit.pas
//
// Abstract:
//
//    Delphi program, to test the FTCI2C.DLL. The FTCI2C.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Inter-Integrated Circuit(I2C) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   09/09/08    Kirke Ashton    Created.
//
unit DateTimeDialogUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls; //, JclStrings;

type
  TDateTimeDialog = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    TimeEdit: TEdit;
    TimeLabel: TLabel;
    DayComboBox: TComboBox;
    DayLabel: TLabel;
    DateEdit: TEdit;
    DateLabel: TLabel;

    function  GetNumbers(StrNumbers, StrType: String; NumberDivider: Char;
                         var Number1: byte; MinNumber1Value, MaxNumber1Value: byte;
                         var Number2: byte; MinNumber2Value, MaxNumber2Value: byte;
                         var Number3: byte; MinNumber3Value, MaxNumber3Value: byte): Boolean;
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);

    function  GetTimeSeconds: Byte;
    function  GetTimeMinutes: Byte;
    function  GetTimeHours: Byte;
    function  GetDayOfWeek: Byte;
    function  GetDayOfMonth: Byte;
    function  GetMonthOfYear: Byte;
    function  GetYearOfCentury: Byte;

  private
    { Private declarations }
    TimeSeconds: Byte;
    TimeMinutes: Byte;
    TimeHours: Byte;
    DayOfWeek: Byte;
    DayOfMonth: Byte;
    MonthOfYear: Byte;
    YearOfCentury: Byte;

  public
    { Public declarations }
end;

const

  NumDaysInWeek = 7;
  DaysInWeek: array[0..(NumDaysInWeek - 1)] of String = ('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday');

var
  DateTimeDialog: TDateTimeDialog;

implementation

{$R *.dfm}

function  TDateTimeDialog.GetNumbers(StrNumbers, StrType: String; NumberDivider: Char;
                                     var Number1: byte; MinNumber1Value, MaxNumber1Value: byte;
                                     var Number2: byte; MinNumber2Value, MaxNumber2Value: byte;
                                     var Number3: byte; MinNumber3Value, MaxNumber3Value: byte): Boolean;
const
  OKOnlyButton = [mbOK];                                    
var
  bNumbersFound: Boolean;
  DivCharSet: TSysCharSet;
  FirstDividerCharIndex, SecondDividerCharIndex, StrLen: Integer;
  Number1Str, Number2Str, Number3Str: String;
begin
  bNumbersFound := False;

  //Include(DivCharSet, NumberDivider);
  DivCharSet := [NumberDivider];

  if True then //(StrCharsCount(StrNumbers, DivCharSet) = 2) then
  begin
    FirstDividerCharIndex := AnsiPos(NumberDivider, StrNumbers);
    SecondDividerCharIndex := LastDelimiter(NumberDivider, StrNumbers);
    StrLen := Length(StrNumbers);

    Number1Str := Copy(StrNumbers, 0, (FirstDividerCharIndex - 1));
    Number2Str := Copy(StrNumbers, (FirstDividerCharIndex + 1), ((SecondDividerCharIndex - FirstDividerCharIndex) - 1));
    Number3Str := Copy(StrNumbers, (SecondDividerCharIndex + 1), StrLen);

    if True then //(StrIsDigit(Number1Str) and StrIsDigit(Number2Str) and StrIsDigit(Number3Str)) then
    begin
      Number1 := StrToInt(Number1Str);
      Number2 := StrToInt(Number2Str);
      Number3 := StrToInt(Number3Str);

      if ((Number1 < MinNumber1Value) or (Number1 > MaxNumber1Value) or
          (Number2 < MinNumber2Value) or (Number2 > MaxNumber2Value) or
          (Number3 < MinNumber3Value) or (Number3 > MaxNumber3Value)) then
        MessageDlg(StrType + ' value out of range.', mtError, OKOnlyButton, 0)
      else
        bNumbersFound := True;
    end
    else
      MessageDlg(StrType + ' illegal character entered.', mtError, OKOnlyButton, 0);
  end
  else
    MessageDlg(StrType + ' string incorrect format.', mtError, OKOnlyButton, 0);

  Result := bNumbersFound;
end;
    
procedure TDateTimeDialog.OKButtonClick(Sender: TObject);
const
  OKOnlyButton = [mbOK];
var
  bDateTimeEntered: Boolean;
begin
  bDateTimeEntered := False;

  if ((Trim(TimeEdit.Text) <> '') and (DayComboBox.ItemIndex <> -1) and (Trim(DateEdit.Text) <> '')) then
  begin
    // Time format ie 15:23:45
    if GetNumbers(Trim(TimeEdit.Text), 'Time', ':', TimeHours, 0, 23, TimeMinutes, 0, 59,
                  TimeSeconds, 0, 59) then
    begin
      DayOfWeek := (DayComboBox.ItemIndex + 1);

      // Date format ie 14/04/05
      if GetNumbers(Trim(DateEdit.Text), 'Date', '/', DayOfMonth, 1, 31, MonthOfYear, 1, 12,
                    YearOfCentury, 0, 99) then
        bDateTimeEntered := True;
    end;
  end
  else
  begin
    if ((Trim(TimeEdit.Text) = '') or (Trim(DateEdit.Text) = '')) then
    begin
      if (Trim(TimeEdit.Text) = '') then
        MessageDlg('Time of day has not been entered.', mtError, OKOnlyButton, 0)
      else
        MessageDlg('Date has not been entered.', mtError, OKOnlyButton, 0);
    end
    else
      MessageDlg('Day of the week has not been selected.', mtError, OKOnlyButton, 0);
  end;

  if bDateTimeEntered then
    ModalResult := mrOK;
end;

procedure TDateTimeDialog.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TDateTimeDialog.FormCreate(Sender: TObject);
var
  DayIndex: Integer;
begin
  TimeSeconds := 0;
  TimeMinutes := 0;
  TimeHours := 0;
  DayOfWeek := 0;

  DayComboBox.Clear;

  for DayIndex := 0 to (NumDaysInWeek - 1) do
    DayComboBox.Items.Append(DaysInWeek[DayIndex]);

  DayOfMonth := 0;
  MonthOfYear := 0;
  YearOfCentury := 0;
end;

function  TDateTimeDialog.GetTimeSeconds: Byte;
begin
  Result := TimeSeconds;
end;

function  TDateTimeDialog.GetTimeMinutes: Byte;
begin
  Result := TimeMinutes;
end;

function  TDateTimeDialog.GetTimeHours: Byte;
begin
  Result := TimeHours;
end;

function  TDateTimeDialog.GetDayOfWeek: Byte;
begin
  Result := DayOfWeek;
end;

function  TDateTimeDialog.GetDayOfMonth: Byte;
begin
  Result := DayOfMonth;
end;

function  TDateTimeDialog.GetMonthOfYear: Byte;
begin
  Result := MonthOfYear;
end;

function  TDateTimeDialog.GetYearOfCentury: Byte;
begin
  Result := YearOfCentury;
end;

end.
